"""
An basic staircase example. Compare it to the _Test class in `twotasks.py`
to see the differences from the "regular" (non-adaptive) approach.

This demo is part of psychopy_ext library.
"""

import sys
import cPickle as pickle

from psychopy import core, visual, data, event
import numpy as np
# pandas does not come by default with PsychoPy but that should not prevent
# people from running the experiment
try:
    import pandas
except:
    pass

from psychopy_ext import exp, ui, stats, plot

# some modules are only available in Python 2.6
try:
    from collections import OrderedDict
except:
    from exp import OrderedDict

import computer
PATHS = exp.set_paths('staircase', computer)


class Staircase(exp.Experiment):
    """
    A perceptual learning experiment
    ================================

    Your task
    ---------

    **Fixate** on the central fixation spot. Two stimuli will briefly flash.
    If they appear to be the same, **hit 'f'**.
    If they appear to be different, **hit 'j'**.
    Please remember to fixate throughout the experiment!

    **Press spacebar to begin.**

    *(Use 'Left Shift + Esc' to exit.)*
    """
    def __init__(self,
        name='exp',
        info=('subjid', 'staircase_'),
        actions='run',
        **kwargs
        ):
        
        super(Staircase, self).__init__(
            name=name,
            info=info,
            actions=actions,
            paths=PATHS,
            computer=computer,
            method='random',
            **kwargs)
        #import pdb; pdb.set_trace()    
        # stimuli orientation
        self.oris = {'attended': 60, 'unattended': 150}        
        self.stim_size = 3.  # in deg
        self.stim_dist = 4.  # from the fixation in polar coords   
        self.ntrials = 30  # number of different trials per session     
        self.convergeto = {'attended': 4, 'unattended': 7}  # to what values each stimulus should converge
        
        self.computer.valid_responses = {'f': 'left', 'j': 'right'}  


    def create_stimuli(self):
        self.create_fixation()
        stim1 = visual.GratingStim(self.win, name='stim1', mask='circle',
                                        sf=2, size=self.stim_size)
        stim2 = visual.GratingStim(self.win, name='stim2', mask='circle',
                                        sf=2, size=self.stim_size)
        self.s = {'fix': self.fixation, 'stim1': stim1, 'stim2': stim2}

    def create_trial(self):
        """Create trial structure
        """
        self.trial = [exp.Event(self, dur=.200,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self, dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim1']],
                                func=self.show_stim),
                      exp.Event(self, dur=.600,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self, dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim2']],
                                func=self.show_stim),
                      exp.Event(self, dur=0,
                                display=self.s['fix'],
                                func=self.wait_until_response)
                     ]    
        
    def create_exp_plan(self):
        self.exp_plan = []  # will be filled up during the runtime
        
    def get_blocks(self):
        blocks = []
        for name in self.oris.keys():
            blocks.append(data.QuestHandler(
                15, 3,
                pThreshold=.63,
                nTrials=self.ntrials,
                minVal=0,
                maxVal=25,
                name=name
                ))
        self.blocks = blocks
        
    def __iter__(self):
        """We need this for loop_trials
        """
        return self

    def next(self):
        """Define each trial's parameters
        """
        this_intensity = self.staircase.next()
        self.thisTrialN = self.staircase.thisTrialN
        
        if np.random.rand() < .5:  # left oriented
            corr_resp = 'left'
            ori_dir = -1
        else:  # right first
            corr_resp = 'right'
            ori_dir = 1

        this_trial = [
            ('trialno', self.thisTrialN),
            ('pos', self.staircase.name),
            ('dir', ori_dir),
            ('oridiff', this_intensity),
            ('onset', ''),
            ('dur', ''),
            ('corr_resp', corr_resp),
            ('subj_resp', ''),
            ('accuracy', ''),
            ('rt', ''),
            ]
        if self.rp['autorun'] > 0:
            auto = self.get_autorun_vals(self.staircase.name,
                    this_intensity, corr_resp)
            this_trial.extend(auto)

        self.this_trial = OrderedDict(this_trial)
        self.exp_plan.append(self.this_trial)
        return self.this_trial        
        
    def show_stim(self):
        if self.this_trial['pos'] == 'attended':
            pos = (-self.stim_dist, 0)
        else:
            pos = (self.stim_dist, 0)

        for stim in self.this_event.display:
            if stim.name == 'stim1':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']])
            if stim.name == 'stim2':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']] + 
                            self.this_trial['dir']*self.this_trial['oridiff'])
            stim.draw()
        self.win.flip()
        self.idle_event(draw_stim=False)

    def get_autorun_vals(self, pos, this_intensity, corr_resp):
        """If autorun, simulate user responses
        """
        def rt(mean):
            add = np.random.normal(mean,scale=.2)/self.rp['autorun']
            return trial_dur + add

        invert_resp = exp.invert_dict(self.computer.valid_responses)
        trial_dur = sum([ev.dur for ev in self.trial])

        thres = np.random.normal(self.convergeto[pos], scale=1)
        thres = max(0, thres)
        invert= {'right': 'left', 'left': 'right'}
        if this_intensity < thres:
            fake_resp = invert[corr_resp]  # wrong response
        else:
            fake_resp = corr_resp  # correct response

        auto = [('autoresp', invert_resp[fake_resp]),
                ('autort', rt(0.500))]
        return auto
        
    def post_trial(self, this_trial, all_keys):
        this_resp = all_keys.pop()
        this_trial['subj_resp'] = self.computer.valid_responses[this_resp[0]]
        acc = exp.signal_det(this_trial['corr_resp'], this_trial['subj_resp'])
        this_trial['accuracy'] = acc
        this_trial['rt'] = this_resp[1]
        if acc == 'correct':
            acc_int = 1
        else:
            acc_int = 0
        self.staircase.addData(acc_int)
        return this_trial
        
    def run_task(self):
        self.setup_task()
        self.before_task()
        
        if not self.rp['no_output']:
            print 'Data will be saved in %s' % self.datafile.filename    
        self.datafile.open()
        for blockno, block in enumerate(self.blocks):
            self.staircase = block
            self.this_blockn = blockno
            self.set_TrialHandler(self.exp_plan)
            self.run_block()
        self.datafile.close()
            
        self.after_task()


class Analysis(object):
    def __init__(self,
                 name='analysis',
                 info={'subjid': 'staircase_'},
                 rp={'all': False},
                 ):
        self.name = name
        self.info = info
        self.rp = rp
        self.paths = PATHS

        if self.rp['all']:
            self._set_all_subj()

    def _set_all_subj(self):
        self.info['subjid'] = ['staircase_%02d' % i for i in range(1,11)]

    def _get_staircase(self):
        pattern = self.paths['data'] + '%s.csv'
        df = exp.get_behav_df(subjid=self.info['subjid'], pattern=pattern)
        df_thres_all = []
        df['thres'] = 0.

        for subj in np.unique(df.subjid):
            subjdf = df[df.subjid == subj]
            for pos in subjdf.pos.unique():
                posdf = subjdf[subjdf.pos == pos]
                st_mod = Staircase()
                st_mod.get_blocks()
                # get staircases from st_mod
                exps = dict((st.name, st) for st in st_mod.blocks)
                exps[pos].importData(posdf.oridiff.astype(float),
                                      np.array(posdf.corr_resp==posdf.subj_resp))
                posdf.thres = exps[pos].mean()
                df_thres_all.append(posdf)
        df_thres = pandas.concat(df_thres_all)
        return df_thres

    def thresholds(self):
        df = self._get_staircase()
        agg_thres = stats.aggregate(df, values='thres', cols='pos',
                                    yerr='subjid')
        plt = plot.Plot()
        plt.plot(agg_thres, kind='bar', title='orientation thresholds',
                 ylabel='orientation threshold, deg')
        print agg_thres.mean()
        plt.show()

        return agg_thres

    def run(self):
        df = self._get_staircase()
        agg_thres = stats.aggregate(df, values='thres', rows='pos', unstack=True)
        agg = stats.aggregate(df, values='oridiff', rows='trialno', cols='pos')
        plt = plot.Plot()
        ax = plt.plot(agg, kind='line', ylabel='step')
        num_trials = len(agg.columns.levels[1])
        thres = agg_thres.mean(1).values[0]
        ax.axhline(y=thres, color = '.2', ls='--', lw=2, marker = 'None')
        print agg_thres
        ax.text(num_trials - num_trials/3., thres+.5,
            'average threshold = %.2f' % thres, fontsize=8)
        plt.show()
        return agg_thres



#!/usr/bin/env python

# Part of the psychopy_ext library
# Copyright 2010-2013 Jonas Kubilius
# The program is distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.

"""Basic command-line and graphic user interface"""

import wx, sys, os, inspect, shutil, subprocess
from types import ModuleType

try:
    from agw import advancedsplash as AS
except ImportError: # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.advancedsplash as AS

from psychopy import core

# some modules are only available in Python 2.6
try:
    from collections import OrderedDict
except:
    from exp import OrderedDict

class Control(object):
    def __init__(self, exp_choices,
                 title='Project',
                 size=(550,400)
                 ):
        """
        Initializes user control interface.

        Determines automatically whether to open a Graphic User Interface (GUI)
        or operate in a Command Line Interface (CLI) based on the number of
        arguments in ``sys.argv``.

        :Args:
            exp_choices
                :class:`~psychopy_ext.ui.Choices`
        :Kwargs:
            title (str, default: 'Project')
                Title of the GUI app window.
            size (tuple of two int, default: (350,400))
                Size of a GUI app.
        """
        # Some basic built-in functions
        try:
            action = sys.argv[1]
        except: # otherwise do standard stuff
            pass
        else:
            recognized = ['--commit','--register','--push']
            if action in recognized:
                self.run_builtin()

        if not isinstance(exp_choices, (list, tuple)):
            exp_choices = [exp_choices]

        if len(sys.argv) > 1:  # command line interface desired
            if sys.argv[1] == 'report':
                report(exp_choices, sys.argv)
            else:
                self.cmd(exp_choices)
        else:
            self.app(exp_choices, title=title, size=size)

    def run_builtin(self, action=None):
        if action is None:
            action = sys.argv[1]
        if action == '--commit':
            try:
                message = sys.argv[2]
            except:
                sys.exit('Please provide a message for committing changes')
            else:
                _repo_action(sys.argv[2:], message=message)
        elif action == '--register':
            try:
                tag = sys.argv[2]
            except:
                sys.exit('Please provide a tag to register')
            else:
                _repo_action(sys.argv[2:], tag=tag)
        elif action == '--push':
            _repo_action(sys.argv[2:])

        sys.exit()

    def cmd(self, exp_choices):
        """
        Heavily stripped-down version of argparse.
        """
        if len(exp_choices) == 1:  # if just a single choice then just take it
            input_mod_alias = None
            input_class_alias = sys.argv[1]
            input_func = sys.argv[2]
            module = exp_choices[0].module
            class_order = exp_choices[0].order
            arg_start = 3
        else:
            input_mod_alias = sys.argv[1]
            input_class_alias = sys.argv[2]
            input_func = sys.argv[3]
            class_order = exp_choices[0].order
            arg_start = 4
            avail_mods = [e.alias for e in exp_choices]
            try:
                idx = avail_mods.index(input_mod_alias)
            except:
                sys.exit("module '%s' not recognized" % input_mod_alias)
            module = exp_choices[idx].module

        if input_mod_alias is not None:
            if input_mod_alias.startswith('-'):
                sys.exit('You have to specify the name of the experiment after %s'
                         % sys.argv[0])
        if input_class_alias.startswith('-') or input_func.startswith('-'):
            sys.exit('You have to specify properly the task you want to run. '
                     "Got '%s %s' instead." % (input_class_alias, input_func))

        if class_order is not None:
            if input_class_alias not in class_order:
                sys.exit('Class %s not available. Choose from:\n%s' %
                    (input_class_alias, ', '.join(class_order)))

        if isinstance(module, str):
            sys.stdout.write('initializing...')
            sys.stdout.flush()
            try:
                __import__(module)
            except:
                raise
            module = sys.modules[module]

        class_aliases, class_obj = _get_classes(module,
            input_class_alias=input_class_alias, class_order=class_order)
        if class_obj is None:
            sys.exit('Class %s not found. Choose from: %s' %
                (input_class_alias, ', '.join([c[0] for c in class_aliases])))

        try:
            class_init = class_obj()
        except:
            #import pdb; pdb.set_trace()
            raise #SyntaxError('This module appears to require some arguments but that'
                #'should not be the case.' )

        info = {}
        rp = {}
        i = arg_start
        if len(sys.argv) > i:
            if sys.argv[i][0] != '-':
                sys.exit('%s should be followed by function arguments '
                'that start with a - or --' % ' '.join(sys.argv[:i]))

            while i < len(sys.argv):
                input_key = sys.argv[i].lstrip('-')
                if input_key == '':
                    sys.exit("There cannot be any '-' just by themselves "
                                  "in the input")
                item = None
                # is input_key among info?
                if hasattr(class_init, 'info'):
                    for key, value in class_init.info.items():
                        if key == input_key or key[0] == input_key:
                            item = (key, value)
                            params = info
                            break
                # is input_key among rp then?
                if item is None and hasattr(class_init, 'rp'):
                    for key, value in class_init.rp.items():
                        if key == input_key or key[0] == input_key:
                            item = (key, value)
                            params = rp
                            break
                # not found?
                if item is None and (hasattr(class_init, 'info') or
                    hasattr(class_init, 'rp')):
                        sys.exit('Argument %s is not recognized' % input_key)
                else: # found!
                    key, value = item
                    if isinstance(value, bool):
                        try:
                            if sys.argv[i+1][0] != '-':
                                input_value = eval(sys.argv[i+1])
                                if not isinstance(input_value, bool):
                                    sys.exit('Expected True/False after %s' %
                                             input_key)
                                else:
                                    params[key] = input_value
                                i += 1
                            else:
                                params[key] = True
                        except IndexError:  # this was the last argument
                            params[key] = True

                    else:
                        try:
                            input_value = sys.argv[i+1]
                        except IndexError:
                            sys.exit('Expected a value after %s but got nothing'
                                 % input_key)

                        if isinstance(value, tuple):
                            if input_value in value:
                                params[key] = input_value
                            else:
                                sys.exit('Value %s is not possible for %s.\n'
                                         'Choose from: %s'
                                         % (input_value, key, value))
                        else:
                            try:
                                ## not safe but fine in this context
                                params[key] = eval(input_value)
                            except:
                                if input_value[0] == '-':
                                    sys.exit('Expected a value after %s but got '
                                                'another argument' % input_key)
                                else:
                                    params[key] = input_value
                        i += 1
                i += 1

        if hasattr(class_init, 'info'):
            class_init.info.update(info)
            for key, value in class_init.info.items():
                if isinstance(value, tuple):
                    class_init.info[key] = value[0]
        if hasattr(class_init, 'rp'):
            class_init.rp.update(rp)
            for key, value in class_init.rp.items():
                if isinstance(value, tuple):
                    class_init.rp[key] = value[0]
        if hasattr(class_init, 'info') and hasattr(class_init, 'rp'):
            class_init = class_obj(info=class_init.info, rp=class_init.rp)
        elif hasattr(class_init, 'info'):
            class_init = class_obj(info=class_init.info)
            class_init.rp = None
        elif hasattr(class_init, 'rp'):
            class_init = class_obj(rp=class_init.rp)
            class_init.info = None
        else:
            class_init = class_obj()
            class_init.info = None
            class_init.rp = None
        sys.stdout.write('\r               ')
        sys.stdout.write('\r')
        sys.stdout.flush()
        try:
            func = getattr(class_init, input_func)
        except AttributeError:
            sys.exit('Function %s not recognized in class %s. Check spelling?' %
                     (input_func, class_obj.__name__))
        else:
            if hasattr(func, '__call__'):
                func()
            else:
                sys.exit('Object %s not callable; is it really a function?' %
                                input_func)

    def app(self, exp_choices=[], title='Experiment', size=None):
        app = MyApp()

        # initial frame with a gauge on it
        frame = wx.Frame(None, title=title, size=size)
        ## Here we create a panel and a listbook on the panel
        panel = wx.Panel(frame)

        if len(exp_choices) > 1:
            lb = Listbook(panel, exp_choices)
            # add pages to the listbook
            for num, choice in enumerate(exp_choices):
                pagepanel = wx.Panel(lb)
                lb.AddPage(pagepanel, choice.name, select=num==0)
            lb.ChangeSelection(0)
            booktype = lb
            panelsizer = wx.BoxSizer()
            panelsizer.Add(booktype, 1,  wx.EXPAND|wx.ALL)
            panel.SetSizer(panelsizer)
        else:  # if there's only one Notebook, don't create a listbook
            setup_page(exp_choices[0], panel)
        # nicely size the entire window
        app.splash.Close()
        panel.Fit()
        if size is None:
            frame.Fit()
        frame.Centre()
        frame.Show()
        app.MainLoop()

    def _type(self, input_key, input_value, value, exp_type):
        if isinstance(value, exp_type):
            try:
                input_value = int(input_value)
            except:
                Exception('Expected %s for %s'
                           % (exp_type, input_key))
            return input_value


def report(exp_choices, args):
    reports = []
    if len(args) == 2:
        argnames = [ch[2] for ch in exp_choices]
    else:
        argnames = args[2:]

    for ch in exp_choices:
        if ch[2] in argnames:
            choice = ch[1]
            if isinstance(choice, str):
                try:
                    __import__(choice)
                except:
                    module = None
                else:
                    module = sys.modules[choice]
            else:
                module = choice
            if module is not None:
                functions = inspect.getmembers(module, inspect.isfunction)
                for name, func in functions:
                    if name == 'report':
                        reports.append((ch[0], func))
                        break
    Report().make(reports)

def _get_classes(module, input_class_alias=None, class_order=None):
    """
    Finds all useable classes in a given module.

    'Usable' means the ones that are not private
    (class name does not start with '_').

    TODO: maybe alse check if upon initialization has info and rp
    """
    if class_order is None:
        class_aliases = []
    else:
        class_aliases = [None] * len(class_order)
    class_obj = None
    found_classes = inspect.getmembers(module, inspect.isclass)
    for name, obj in found_classes:
        init_vars = inspect.getargspec(obj.__init__)
        try:
            #init_vars.args.index('info')
            #init_vars.args.index('rp')
            init_vars.args.index('name')
        except:
            pass
        else:
            if name[0] != '_':  # avoid private classes
                class_alias = _get_class_alias(module, obj)
                if class_alias == input_class_alias:
                    class_obj = obj
                if class_order is not None:
                    try:
                        idx = class_order.index(class_alias)
                        class_aliases[idx] = (class_alias, obj)
                    except:
                        pass
                else:
                    class_aliases.append((class_alias, obj))
    # if some class not found; get rid of it
    class_aliases = [c for c in class_aliases if c is not None]
    return class_aliases, class_obj

def _get_class_alias(module, obj):
    # make sure this obj is defined in module rather than imported
    if obj.__module__ == module.__name__:
        try:
            init_vars = inspect.getargspec(obj.__init__)
        except:
            pass
        else:
            try:  # must have a name, info, and rp
                nameidx = init_vars.args.index('name')
            except:
                pass
            else:
                class_alias = init_vars.defaults[nameidx - len(init_vars.args)]
                return class_alias

def _get_methods(myclass):
    """
    Finds all functions inside a class that are callable without any parameters.
    """
    methods = []
    for name, method in inspect.getmembers(myclass, inspect.ismethod):
        if name[0] != '_':  # avoid private methods
            mvars = inspect.getargspec(method)
            if len(mvars.args) == 1:  # avoid methods with input variables
                if mvars.args[0] == 'self':
                    methods.append((name, method))
    return methods

def _get_methods_byname(myclass):
    if hasattr(myclass, 'actions'):
        if myclass.actions is not None:
            if isinstance(myclass.actions, str):
                actions = [myclass.actions]
            else:
                actions = myclass.actions
            methods = []
            for action in actions:
                try:
                    func = getattr(myclass, action)
                except AttributeError:
                    pass
                else:
                    methods.append([action, func])
            if len(methods) == 0:
                return _get_methods(myclass)
            else:
                return methods
        else:
            return _get_methods(myclass)
    else:
        return _get_methods(myclass)


class MyApp(wx.App):

    def __init__(self):
        super(MyApp, self).__init__(redirect=False)
        path = os.path.join(os.path.dirname(__file__), 'importing.png')
        image = wx.Bitmap(path, wx.BITMAP_TYPE_PNG)
        self.splash = AS.AdvancedSplash(None, bitmap=image,
                                        style=AS.AS_NOTIMEOUT|wx.FRAME_SHAPED)
        self.splash.SetText(' ')  # bitmap doesn't show up without this
        wx.Yield()  # linux wants this line


class StaticBox(wx.StaticBox):
    def __init__(self, parent, label='', content=None):
        """
        Partially taken from :class:`psychopy.gui.Dlg`
        """
        wx.StaticBox.__init__(self, parent, label=label)
        self.sizer = wx.StaticBoxSizer(self)
        grid = wx.FlexGridSizer(rows=len(content), cols=2)
        self.inputFields = []
        for label, initial in content.items():
            #import pdb; pdb.set_trace()
            #create label
            labelLength = wx.Size(9*len(label)+16,25)#was 8*until v0.91.4
            inputLabel = wx.StaticText(parent,-1,label,
                                            size=labelLength,
                                           )
            #if len(color): inputLabel.SetForegroundColour(color)
            grid.Add(inputLabel, 1, wx.ALIGN_LEFT)
            #create input control
            if isinstance(initial, bool):
                inputBox = wx.CheckBox(parent, -1)
                inputBox.SetValue(initial)
            elif isinstance(initial, int):
                inputBox = wx.SpinCtrl(parent, size=(60, -1), initial=initial)
            elif isinstance(initial, tuple):
                inputBox = wx.Choice(parent, -1,
                            choices=[str(option) for option in initial])
                ## Somewhat dirty hack that allows us to treat the choice just like
                ## an input box when retrieving the data
                inputBox.GetValue = inputBox.GetStringSelection
                #if initial in choices:
                    #initial = choices.index(initial)
                #else:
                    #initial = 0
                inputBox.SetSelection(0)
            else:
                inputLength = wx.Size(max(50, 9*len(unicode(initial))+16), 25)
                inputBox = wx.TextCtrl(parent,-1,unicode(initial),size=inputLength)

            #if len(color): inputBox.SetForegroundColour(color)
            #if len(tip): inputBox.SetToolTip(wx.ToolTip(tip))
            self.inputFields.append(inputBox)#store this to get data back on button click
            grid.Add(inputBox, 1, wx.ALIGN_LEFT)

        self.sizer.Add(grid)
        #self.SetSizer(self.sizer)

class Page(wx.Panel):
    """
    Creates a page inside a Notebook with two boxes, Information and Parameters,
    corresponding to info and rp in :class:`exp.Experiment`, and
    buttons which, when clicked, runs a corresponding method.
    """
    def __init__(self, parent, class_obj, alias, class_alias):
        wx.Panel.__init__(self, parent, -1)
        self.class_obj = class_obj
        self.alias = alias
        self.class_alias = class_alias

        class_init = class_obj()
        if not hasattr(class_init, 'info'):
            class_init.info = None
        if not hasattr(class_init, 'rp'):
            class_init.rp = None
        if class_init.info is not None:
            self.sb1 = StaticBox(self, label="Information",
                content=class_init.info)
        if class_init.rp is not None:
            self.sb2 = StaticBox(self, label="Parameters",
                content=class_init.rp)

        # generate buttons
        # each button launches a function in a given class
        #actions = _get_methods(class_init)
        actions = _get_methods_byname(class_init)
        # buttons will sit on a grid of 2 columns and as many rows as necessary
        buttons = wx.FlexGridSizer(rows=0, cols=2)
        add = False
        for i, (label, action) in enumerate(actions):
            if hasattr(class_init, 'actions'):
                if class_init.actions is not None:
                    if isinstance(class_init.actions, str):
                        class_init.actions = [class_init.actions]
                    if label in class_init.actions:
                        add = True
            else:
                add = True
            if add:
                run = wx.Button(self, label=label, size=(150, 30))
                buttons.Add(run, 1)
                run.info = class_init.info  # when clicked, what to do
                run.rp = class_init.rp
                run.action = label
                run.Bind(wx.EVT_BUTTON, self.OnButtonClick)
                if i==0: run.SetFocus()

        pagesizer = wx.BoxSizer(wx.VERTICAL)
        # place the two boxes for entering information
        if class_init.info is not None:
            pagesizer.Add(self.sb1.sizer)
        if class_init.rp is not None:
            pagesizer.Add(self.sb2.sizer)
        # put the buttons in the bottom
        pagesizer.Add(buttons, 1, wx.ALL|wx.ALIGN_LEFT)
        self.SetSizer(pagesizer)

    def OnButtonClick(self, event):
        #module = event.GetEventObject().module
        ## first update info and rp
        button = event.GetEventObject()
        if button.info is not None:
            for key, field in zip(button.info.keys(), self.sb1.inputFields):
                button.info[key] = field.GetValue()
        if button.rp is not None:
            for key, field in zip(button.rp.keys(), self.sb2.inputFields):
                button.rp[key] = field.GetValue()

        # taken from PsychoPy's builder.py
        #scriptProcess = wx.Process(self) #self is the parent (which will receive an event when the process ends)
        #scriptProcess.Redirect()#builder will receive the stdout/stdin
        opts = [self.alias, self.class_alias, button.GetLabelText()]
        info = []
        for k,v in button.info.items():
            try:
                gaps = len(v.split(' '))
            except:
                info.append('--%s %s' %(k,v))
            else:
                if gaps > 1:
                    info.append('--%s "%s"' %(k,v))
                else:
                    info.append('--%s %s' %(k,v))
        rp=[]
        for k,v in button.rp.items():
            try:
                gaps = len(v.split(' '))
            except:
                rp.append('--%s %s' %(k,v))
            else:
                if len(v.split(' ')) > 1:
                    rp.append('--%s "%s"' %(k,v))
                else:
                    rp.append('--%s %s' %(k,v))
        argv = ' '.join(opts + info + rp)

        # -u forces stdin, stdout and stderr to be totally unbuffered
        if sys.platform=='win32':
            # the quotes allow file paths with spaces
            command = '"%s" -u "%s" %s' % (sys.executable, sys.argv[0], argv)
            # wx.Execute(command, wx.EXEC_SYNC | wx.EXEC_NOHIDE)
        else:
            # for unix this signifies a space in a filename
            python_exec = sys.executable.replace(' ','\ ')
            # the quotes would break a unix system command
            command = '"%s" -u "%s" %s' % (sys.executable, sys.argv[0], argv)
            # wx.Execute(command, wx.EXEC_SYNC| wx.EXEC_MAKE_GROUP_LEADER)

        subprocess.call(command, shell=False)

        #try:
            #class_init = self.class_obj(info=button.info, rp=button.rp)
        #except:
            #try:
                #class_init = self.class_obj(info=button.info)
            #except:
                #try:
                    #class_init = self.class_obj(rp=button.rp)
                #except:
                    #class_init = self.class_obj()
        #func = getattr(class_init, button.action)
        #func()  # FIX: would be nice to keep it open at the end of the exp


class Listbook(wx.Listbook):
    """
    Listbook class
    """
    def __init__(self, parent, exp_choices):
        wx.Listbook.__init__(self, parent, id=wx.ID_ANY)
        self.exp_choices = exp_choices
        self.ready = []
        self.Bind(wx.EVT_LISTBOOK_PAGE_CHANGING, self.OnPageChanging)

    def OnPageChanging(self, event):
        new = event.GetSelection()
        if new not in self.ready:
            setup_page(self.exp_choices[new], self.GetPage(new))
            self.ready.append(new)
        #wx.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))
        #wx.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

def setup_page(choice, pagepanel):
    """
    Creates a :class:`Page` inside a :class:`Notebook`.

    :Args:
        - choice (tuple)
            A tuple of (name, module path, module alias)
        - pagepanel
    """
    if isinstance(choice.module, str):
        __import__(choice.module)
        class_aliases, class_obj = _get_classes(sys.modules[choice.module],
            class_order=choice.order)
    else:
        class_aliases, class_obj = _get_classes(choice.module, class_order=choice.order)
    nb = wx.Notebook(pagepanel)
    for class_alias, class_obj in class_aliases:
        nb.AddPage(Page(nb, class_obj, choice.alias, class_alias), class_alias)
    panelsizer = wx.BoxSizer()
    panelsizer.Add(nb, 1,  wx.EXPAND|wx.ALL)
    pagepanel.SetSizer(panelsizer)
    pagepanel.Layout()
    pagepanel.Fit()

def _detect_rev():
    """
    Detects revision control system.

    Recognizes: git, hg
    """
    revs = ['git', 'hg']
    for rev in revs:
        try:
            out, err = core.shellCall(rev + ' status', stderr=True)
        except:  # revision control is not installed
            pass
        else:
            if err[:5] not in ['abort', 'fatal']:
                return rev

def _repo_action(cmd, **kwargs):
    """
    Detects revision control system and performs a specified action.

    Currently supported: committing changes, tagging the current version of the
    repository (registration), and pushing.

    'Registration' is inspired by the `Open Science Framework
    <http://openscienceframework.org/>`_. Useful when you start running
    participants so that you can always go back to that version.
    """
    rev = _detect_rev()
    if rev == 'hg':
        if cmd == 'push':
            call = 'hg push'
        elif cmd == 'commit':
            if 'message' in kwargs:
                call = 'hg commit -A -m "%s"' % kwargs['message']
            else:
                raise Exception('Please provide a message for committing changes')
        elif cmd == 'register':
            if 'tag' in kwargs:
                call = 'hg tag %s' % kwargs['tag']
            else:
                raise Exception('Please provide a tag to register')
        else:
            raise Exception("%s is not supported for %s yet" % (rev, cmd))

    elif rev == 'git':
        if cmd == 'push':
            call = 'git push'
        elif cmd == 'commit':
            if 'message' in kwargs:
                call = 'git commit -am "%s"' % kwargs['message']
            else:
                raise Exception('Please provide a message for committing changes')
        else:
            raise Exception("%s is not supported for %s yet" % (rev, cmd))

    elif rev is None:
        raise Exception("no revision control detected")
    else:
        raise Exception("%s is not supported for %s yet" % (rev, cmd))

    out, err = core.shellCall(call, stderr=True)
    call = '$ ' + call
    write = [call]
    if out != '':
        write.append(out)
    if err != '':
        write.append(err)
    sys.stdout.write('\n'.join(write) + '\n')
    return call, out, err


class Choices(object):

    def __init__(self, module, name='', alias=None, order=None):
        """
        Holds choices for calling experiments.

        :Args:
            module (str or module)
                The module you want to call. If your script is in
                'scripts/main.py', then module should be 'scripts.main'.
                You can also give the module itself if you already have
                it imported::

                    import scripts.main
                    exp_choices = Choices(scripts.main)

        :Kwargs:
            - name (str, default: '')
                Name of the experiment.
            - alias (str, default: None)
                For CLI: alias for calling this experiment. If *None*,
                will be inferred from ``module``.
            - order (list, default: None)
                For GUI: Order of tabs (classes).
        """
        ## if direct path to the experiment
        #if isinstance(exp_choices, str) or isinstance(exp_choices, ModuleType):
            #exp_choices = [('Experiment', exp_choices, 'main')]
        #elif len(exp_choices) == 0:
            #sys.exit('exp_choices is not supposed to be empty')

        #choices = []
        #for choice in exp_choices:
            #if len(choice) == 0:
                #sys.exit('Please give at least a path to the experiment')
            #elif len(choice) == 1:  # path to experiment is given
                #choices.append(('Experiment', choice, choice[1], None))
            #elif len(choice) == 2:  # if 'scripts.main', then cli call alias is 'main'
                #choices.append(choice + (choice[1].split('.',1)[1], None))
            #elif len(choice) == 3:
                #choices.append(choice + (None,))
            #else:
                #choices.append(choice)
        #exp_choices = choices

        self.module = module
        self.name = name

        if alias is None:
            try:
                self.alias = module.split('.',1)[1]
            except:
                import pdb; pdb.set_trace()
                self.alias = module.__name__
        else:
            self.alias = alias

        self.order = order


class Report(object):

    def __init__(self, output='html', path='report'):
        self.path = path+'/'
        #if not os.path.isdir(self.paths['report']):
            #os.makedirs(self.paths['report'])
        #self.rp['plot'] = False
        #self.rp['saveplot'] = True
        #self.rp['html'] = True

    def write(self, text):
        self.htmlfile.write(text)

    def make(self, reports):
        if not os.path.isdir(self.path):
            os.makedirs(self.path)
        else:
            for root, dirs, files in os.walk(self.path):
                #for f in files:
                    #try:
                          #os.unlink(os.path.join(root, f))
                    #except:
                        #pass
                for d in dirs:
                    try:
                          shutil.rmtree(os.path.join(root, d))
                    except:
                        pass

        src = os.path.dirname(__file__)
        src = os.path.join(src, 'resources/')
        import glob
        for f in glob.glob(src+'*'):
            if os.path.isfile(f):
                if os.path.basename(f) != 'index.html':
                    shutil.copy2(f)
            else:
                dst = os.path.join(self.path, os.path.basename(f))
                shutil.copytree(f, dst,
                    ignore=shutil.ignore_patterns('index.html'))
        with open(src + 'index.html', 'rb') as tmp:
            template = tmp.read().split('####REPLACE####')
        self.htmlfile = open(self.path + 'index.html', 'wb')
        self.write(template[0])
        for name, report in reports:
            self.writeh(name, h='h1')
            report(self)

        self.write(template[1])
        self.htmlfile.close()

    def writeimg(self, names, caption=None, plt=None, win=None):
        if isinstance(names, str):
            names = [names]
        fname = '_'.join(names) + '.png'
        imgdir = self.path + self.imgpath
        if plt is not None or win is not None:
            if not os.path.isdir(imgdir):
                os.makedirs(imgdir)
            if plt is not None:
                plt.savefig(imgdir + fname, bbox_inches='tight')
            elif win is not None:
                win.saveMovieFrames(imgdir + fname)
        if caption is None:
            caption = ' '.join(names)
        self.htmlfile.write(
            '<figure>\n'
            '    <img src="%s" />\n'
            '    <figcaption><strong>Figure.</strong> %s</figcaption>\n'
            '</figure>\n' % (self.imgpath + fname, caption)
            )

    def writeh(self, text, h='h1'):
        self.htmlfile.write('<%s>%s</%s>\n' % (h, text, h))





#!/usr/bin/env python

"""
Computer configuration file
===========================

Specify default settings for all computers where you run your experiment such
as a monitor size or root path to storing data. This is intended as a more
portable and extended version of PsychoPy's MonitorCenter.

A computer is recognized by its mac address which is dependent on its
hardware and by its name. In the future versions of psychopy_ext,
if anything in the hardware changes, you'll see a warning.

# TODO: split computer configuration and defaults possibly by moving to a
config file

"""

import uuid, platform

recognized = True
# computer defaults
root = '.'  # means store output files here
stereo = False  # not like in Psychopy; this merely creates two Windows
default_keys = {'exit': ('lshift', 'escape'),  # key combination to exit
                'trigger': 'space'}  # hit to start the experiment
valid_responses = {'0': 0, '1': 1}  # organized as input value: output value
# monitor defaults
distance = 80
width = 37.5
# window defaults
screen = 0  # default screen is 0
view_scale = (1, 1)

# Get computer properties
# Computer is recognized by its mac address
mac = uuid.getnode()
system = platform.uname()[0]
name = platform.uname()[1]

if mac == 153254424819 and system == 'Linux':  # Lab computer, Ubuntu booted
    distance = 80
    width = 37.5
    root = '/media/qbilius/Data/data/'

elif mac == 153254424819 and system == 'Windows':  # Lab computer, Windows booted
    root = 'D:/data/'

elif mac == 145320949993177:  # fMRI computer
    distance = 127
    width = 60
    view_scale = [1,-1]  # top-bottom inverted
    default_keys['trigger'] = 5
    valid_responses = {'9': 0, '8': 1, '7': 2, '6': 3}

else:
    recognized = False



import numpy as np
from psychopy import visual, monitors
from .. import exp
import unittest

# some modules are only available in Python 2.6
try:
    from collections import OrderedDict
except:
    from exp import OrderedDict

exp.default_computer.recognized = True  # so that tests can proceed
PATHS = exp.set_paths('', exp.default_computer)

class TestExp(unittest.TestCase):

    def test_setpaths(self):
        exp.set_paths('', exp.default_computer)
        exp.set_paths()

    def test_exp(self):
        thisexp = MyExp(rp={'no_output': True, 'debug': True, 'unittest': True})
        with self.assertRaises(SystemExit):
            thisexp.run()

    def test_ThickShapeStim(self):
        monitor = monitors.Monitor('test', distance=57, width=37)
        monitor.setSizePix((1028, 768))
        win = visual.Window([128,128], monitor=monitor)
        line = exp.ThickShapeStim(win)
        line.draw()
        line.setOri(10)
        line.setPos((10,10))
        line.setVertices(value=[(-.5,0),(0,.5),(.5,0),(.7,.9)])
        line.draw()
        win.close()
        
    def test_GroupStim(self):
        win = visual.Window([128,128])
        line1 = visual.ShapeStim(win)
        line2 = visual.ShapeStim(win)
        group1 = exp.GroupStim(stimuli=line1, name='group1')
        group2 = exp.GroupStim(stimuli=[line1, line2])
        group1.draw()
        group1.setPos((10, 8))
        group2.setOri(10)
        group1.draw()
        group2.draw()
        win.close()
        
    def test_invert_dict(self):
        d = {1: 2, -3: 4, 0: 'a'}
        invd = exp.invert_dict(d)
        self.assertEqual(invd, OrderedDict([(2,1), (4,-3), ('a',0)]))
        
    def test_other(self):
        self.assertEqual(exp.signal_det('5', '5'), 'correct')
        exp.get_mon_sizes()        


class MyExp(exp.Experiment):
    """
    Test experiment.
    """
    def __init__(self, name='exp', **kwargs):
        # initialize the default Experiment class with our parameters
        super(MyExp, self).__init__(name=name, paths=PATHS,
                                    computer=exp.default_computer, **kwargs)
        self.computer.valid_responses = {'1': 0, 'd': 1}

    def create_stimuli(self):
        """Define stimuli
        """
        self.create_fixation()
        stim1 = visual.ShapeStim(self.win)
        stim2 = visual.ShapeStim(self.win)

        self.s = {
            'fix': self.fixation,
            'stim1': stim1,
            'both': exp.GroupStim(stimuli=[stim1, stim2], name='both')
            }

    def create_trial(self):
        """Create trial structure
        """
        self.trial = [exp.Event(self,
                                dur=0.300,  # in seconds
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self,
                                dur=0,  # this means present until response
                                display=[self.s['stim1'], self.s['both']],
                                func=self.wait_until_response),
                      exp.Event(self,
                                dur=.300,
                                display=self.s['fix'],
                                func=self.feedback)
                     ]

    def create_exp_plan(self):
        """Define each trial's parameters
        """
        exp_plan = []
        for cond in range(8):
            exp_plan.append(OrderedDict([
                ('cond', cond),
                ('onset', ''),
                ('dur', ''),
                ('corr_resp', 1),
                ('subj_resp', ''),
                ('accuracy', ''),
                ('rt', ''),
                ]))
        self.exp_plan = exp_plan


if __name__ == '__main__':
    unittest.main()



"""
A perceptual learning experiment
================================

This demo combined twotasks.py and staircase.py into a single quite
complex experiment.

This demo is part of psychopy_ext library.
"""

import sys
import cPickle as pickle

from psychopy import core, visual, data, event
import numpy as np
# pandas does not come by default with PsychoPy but that should not prevent
# people from running the experiment
try:
    import pandas
except:
    pass

from psychopy_ext import exp, ui, stats, plot

# some modules are only available in Python 2.6
try:
    from collections import OrderedDict
except:
    from exp import OrderedDict

import computer
PATHS = exp.set_paths('perclearn', computer)


class PercLearn(exp.Experiment):
    """
    A perceptual learning experiment
    ================================

    This experiment is composed of **four parts**. Each part consists of
    **training** for 1 min (for presentation purposes only) and
    **testing** for 5 min.

    **Press spacebar to continue.**

    *(Use 'Left Shift + Esc' to exit.)*
    """
    def __init__(self,
        name='exp',
        info=('subjid', 'perclearn_'),
        rp=OrderedDict([
            ('phase', ('both', 'train', 'test')),
            ('kind', ('2AFC', 'QUEST')),
            ('practice', False),
            ])
        ):
        super(PercLearn, self).__init__(name=name, info=info, rp=rp,
                actions='run', paths=PATHS, computer=computer)
        if self.rp['practice']:
            self.rp['no_output'] = True

        self.nsessions = 3 # number of sessions
        if self.rp['kind'] == '2AFC':
            self.tasks = [_Train, _Test2AFC]
        else:
            self.tasks = [_Train, _TestQuest]

        # stimuli orientation
        self.oris = {'attended': 60, 'unattended': 150}
        self.stim_size = 3.  # in deg
        self.stim_dist = 4.  # from the fixation in polar coords

    def run(self):
        self.setup()
        self.before_exp()
        for expno in range(self.nsessions):
            self.show_text('Session %d' % (expno+1), auto=1)
            if self.rp['phase'] in ['train','both']:
                self.tasks[0](self, session=expno+1).run_task()
            if self.rp['phase'] in ['test', 'both']:
                self.tasks[1](self, session=expno+1).run_task()
            pause_instr = ('End of session %d.\n'
                           'When ready, press spacebar to continue.'
                            % (expno+1))
            if expno != self.nsessions - 1:
                self.show_text(pause_instr)
        self.after_exp()

        if self.rp['register']:
            self.register()
        elif self.rp['push']:
            self.commitpush()
        self.quit()

class _Train(exp.Task):
    """
    Training
    ========

    Your task
    ---------

    **Fixate** on the central fixation spot.
    Attend to stimulus on the **left**.
    When you notice a decrease in its contrast, **press 'j'**.
    Please remember to fixate throughout the experiment.

    **Press spacebar to begin.**

    *(Use 'Left Shift + Esc' to exit.)*
    """

    def __init__(self, parent, session=1):

        data_fname = parent.paths['data'] + parent.info['subjid'] + '_train.csv'
        super(_Train, self).__init__(
            parent,
            method='random',
            data_fname=data_fname
            )
        self.session = session
        self.computer.valid_responses = {'j': 1}

        self.oris = parent.oris
        self.stim_size = parent.stim_size
        self.stim_dist = parent.stim_dist

        self.ntrials = 75  # number of trials per session
        self.nblocks = 2
        self.task_rate = .1  # how often lower contrast stimulus should come up
        self.low_contrast = .5

        self.anl = Analysis(info=self.parent.info)

    def create_stimuli(self):
        # Define stimuli
        self.create_fixation()
        self.s = {'fix': self.fixation}
        self.s['attended'] = visual.GratingStim(self.win, name='attended',
                                mask='circle', sf=2, size=self.stim_size,
                                pos=(-self.stim_dist,0),
                                ori=self.oris['attended'])
        self.s['unattended'] = visual.GratingStim(self.win, name='unattended',
                                mask='circle', sf=2, size=self.stim_size,
                                pos=(self.stim_dist,0),
                                ori=self.oris['unattended'])

    def create_trial(self):
        """Create trial structure
        """
        allstim = [self.s['fix'], self.s['attended'], self.s['unattended']]

        self.trial = [exp.Event(self,
                                dur=.200,  # in seconds
                                display=allstim,
                                func=self.show_stim),
                      exp.Event(self,
                                dur=.200,
                                display=self.s['fix'],
                                func=self.idle_event)
                     ]

    def create_exp_plan(self):
        """Define each trial's parameters
        """
        exp_plan = []
        for block in range(self.nblocks):
            for trial in range(self.ntrials):  # repeat the defined number of times
                if trial < self.ntrials * self.task_rate:
                    cond = 'low'
                    corr_resp = 1
                else:
                    cond = 'high'
                    corr_resp = ''
                exp_plan.append(OrderedDict([
                    ('session', self.session),
                    ('block', block),
                    ('cond', cond),
                    ('onset', ''),
                    ('dur', ''),
                    ('corr_resp', corr_resp),
                    ('subj_resp', ''),
                    ('rt', ''),
                    ]))
        self.exp_plan = exp_plan

    #def set_autorun(self, trial_list):
        #def rt(mean):
            #add = np.random.normal(mean,scale=.2) / self.rp['autorun']
            #return trial[0].dur + add

        #invert_resp = exp.invert_dict(self.computer.valid_responses)
        #trial_dur = sum([ev.dur for ev in self.trial])

        #for trial in trialList:
            #trial['autort'] = ''
            #trial['autoresp'] = ''
            #if trial['corr_resp'] == 1:
                #acc = [.05,.95]
                #resp_ind = self.weighted_choice(weights=acc)
                #if resp_ind == 1:
                    #trial['autort'] = rt(.8)
                    ##add = int(np.floor(thisrt/self.trialDur)) + 1
                    #trial['autoresp'] = 1
        #return trialList

    #def get_auto_resp(self, trial):
        #resp_win = 5
        #start = max(0, self.thisTrialN-resp_win+1)
        #end = self.thisTrialN+1#min(self.thisTrialN+1, len(self.trialList))
        ##trials = self.trialList[self.sequenceIndices[start:end].T[0]])
        #resp = ''
        #rt = ''
        ##import pdb; pdb.set_trace()
        ##trialDur = self.trialDur * self.runParams['autorun']
        #for i, idx in enumerate(self.sequenceIndices[start:end]):
            #tr = self.trialList[idx[0]]
            #if tr['autoRT'] != '':
                #if i == end-start-1:
                    #trialDur = self.trialDur
                #else:
                    #trialDur = self.trialDur * self.runParams['autorun']
                #offset = int(np.floor(tr['autoRT']/trialDur))+1
                #if offset == end-start-i:
                    #rt = tr['autoRT'] % trialDur
                    #if i != end-start-1:
                        #rt /= self.runParams['autorun']
                    #try:
                        #resp += 1
                    #except:
                        #resp = 1
            ##tr['autoRT']
        ##import pdb; pdb.set_trace()
        ###for self.triaList[

        ##if trial['corrResp'] == 1:
            ##acc = [.05,.95]
            ##resp_ind = self.weighted_sample(acc)
            ##if resp_ind == 1:
                ###thisrt = rt(.8)  # fast responses
                ##add = int(np.floor(thisrt/self.trialDur)) + 1

        ##import pdb; pdb.set_trace()
        ##for i, trial in enumerate(trialList):
            ##trial['autoResp'] = resps_acc[i]
            ##trial['autoRT'] = resps_rt[i]

        #return (resp, rt)#List

    def set_autorun(self, exp_plan):
        def rt(mean):
            add = np.random.normal(mean,scale=.2) / self.rp['autorun']
            return self.trial[0].dur + add

        for trial in exp_plan:
            trial['autort'] = ''
            trial['autoresp'] = ''
            if trial['corr_resp'] == 1:
                acc = [.10,.90]
                resp_ind = exp.weighted_choice(weights=acc)
                if resp_ind == 1:
                    trial['autort'] = rt(.8)
                    trial['autoresp'] = 1
        return exp_plan

    def get_auto_resp(self, trial):
        """
        In case of autorun, simulates user responses.

        This is very complicated but the idea is to simulate responses
        around the time the stimulus was presented.
        """
        resp_win = 5  # how many trials to consider prior to this one
                      # as valid user inputs
                      # we don't want to accept inputs too late after
                      # contrast change as "correct"
        start = max(0, self.thisTrialN-resp_win+1)
        end = self.thisTrialN+1
        resp = ''
        rt = ''
        # go through trials in resp_win and assign responses
        for i, idx in enumerate(self.sequenceIndices[start:end]):
            tr = self.trialList[idx[0]]  # get the ith trial
            if tr['autort'] != '':  # was there an auto response then?
                trial_dur = sum([ev.dur for ev in self.trial])
                if i != end-start-1:
                    trial_dur *= self.rp['autorun']
                # after how many trials should autoresp become subj_resp?
                offset = int(np.floor(tr['autort']/trial_dur))+1
                # are we there yet or will subj_resp happen later?
                if offset == end-start-i:
                    rt = tr['autort'] % trial_dur
                    if i != end-start-1:
                        rt /= self.rp['autorun']
                    try:
                        resp += 1
                    except:
                        resp = 1
        return (resp, rt)

    def show_stim(self):
        if self.this_trial['cond'] == 'low':
            contrast = self.low_contrast
        else:
            contrast = 1
        for stim in self.this_event.display:
            if stim.name == 'attended':
                stim.setContrast(contrast)
            stim.draw()
        self.win.flip()

        event_keys = self.idle_event(draw_stim=False)

        return event_keys

    def post_trial(self, this_trial, all_keys):
        """ What to do after a trial is over.
        """
        if self.rp['autorun'] > 0:
            allKeys = [self.get_auto_resp(this_trial)]

        if len(all_keys) > 0:
            if all_keys[0][0] != '':
                this_trial['subj_resp'] = len(all_keys)
            else:
                this_trial['subj_resp'] = ''
            this_trial['rt'] = all_keys[-1][1]
        else:
            this_trial['subj_resp'] = ''
            this_trial['rt'] = ''

        return this_trial

    def before_task(self):
        """We slightly redefine the default function so that full
        instructions are shown the first time round.
        """
        if self.session == 1:
            super(_Train, self).before_task()
        else:
            text = '''
            Training, session %d
            --------------------

            (decrease in its contrast: **press 'j'**)
            '''
            super(_Train, self).before_task(text=text % self.session)

    def after_task(self):
        acc = self.anl.train_feedback(self.session)
        pause_instr = 'Your accuracy is %d%%.' % (100*acc)
        super(_Train, self).after_task(text=pause_instr)


class _BaseTest(exp.Task):
    """
    Testing
    =======

    Your task
    ---------

    **Fixate** on the central fixation spot. Two stimuli will briefly flash.
    If they appear to be the **same, hit 'f'**.
    If they appear to be **different, hit 'j'**.
    Please remember to fixate throughout the experiment!

    **Press spacebar to begin.**

    *(Use 'Left Shift + Esc' to exit.)*
    """
    def __init__(self, parent, session=1):
        data_fname = (parent.paths['data'] + parent.info['subjid'] +
                                                           '_test.csv')
        super(_BaseTest, self).__init__(
            parent,
            method='random',
            data_fname=data_fname
            )
        self.session = session
        self.oris = parent.oris
        self.stim_size = parent.stim_size
        self.stim_dist = parent.stim_dist

        self.computer.valid_responses = {'f': 'same', 'j': 'diff'}


    def create_stimuli(self):
        self.create_fixation()
        stim1 = visual.GratingStim(self.win, name='stim1', mask='circle',
                                        sf=2, size=self.stim_size)
        stim2 = visual.GratingStim(self.win, name='stim2', mask='circle',
                                        sf=2, size=self.stim_size)
        self.s = {'fix': self.fixation, 'stim1': stim1, 'stim2': stim2}

    def create_trial(self):
        """Create trial structure
        """
        self.trial = [exp.Event(self,
                                dur=.200,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self,
                                dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim1']],
                                func=self.show_stim),
                      exp.Event(self,
                                dur=.600,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self,
                                dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim2']],
                                func=self.show_stim),
                      exp.Event(self,
                                dur=0,
                                display=self.s['fix'],
                                func=self.wait_until_response)
                     ]
        if self.rp['practice']:
            self.trial.append(exp.Event(self,
                                        dur=0.200,
                                        display=self.s['fix'],
                                        func=self.feedback))

    def show_stim(self):
        if self.this_trial['pos'] == 'attended':
            pos = (-self.stim_dist, 0)
        else:
            pos = (self.stim_dist, 0)

        for stim in self.this_event.display:
            if stim.name == 'stim1':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']])
            if stim.name == 'stim2':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']] +
                            self.this_trial['dir']*self.this_trial['oridiff'])
            stim.draw()
        self.win.flip()

        self.idle_event(draw_stim=False)

    def before_task(self):
        """We slightly redefine the default function so that full
        instructions are shown the first time round.
        """
        if self.session == 1:
            super(_BaseTest, self).before_task(text=_BaseTest.__doc__)
        else:
            text = '''
            Testing, session %d
            -------------------

            (same stimuli: **hit 'f'**, different stimuli: **hit 'j'**)
            '''
            super(_BaseTest, self).before_task(text=text % self.session)


class _TestQuest(_BaseTest):

    def __init__(self, *args, **kwargs):
        """
        Runs a QUEST procedure.
        """
        super(_TestQuest, self).__init__(*args, **kwargs)
        self.ntrials_diff = 30  # number of different trials per session
        self.ntrials_same = 5 # number of same trials per session
        self.convergeto = {'attended': 4, 'unattended': 7}  # to what values each stimulus should converge

    def __iter__(self):
        """We need this for loop_trials
        """
        return self

    def next(self):
        """Define each trial's parameters
        """
        if np.random.rand() < .5:  # left oriented
            ori_dir = -1
        else:  # right first
            ori_dir = 1
        prop = self.ntrials_same / float(self.ntrials_diff + self.ntrials_same)

        # if both have the same orientation
        if np.random.rand() < prop and self.count_same < self.ntrials_same:
            this_intensity = 0
            corr_resp = 'same'
            self.count_same += 1
            trialno = self.count_same

        else: # different orientations
            # get the next suggested intensity and condition
            # staircase is automatically updated to thisCondition
            self.staircase._checkFinished()

            if self.staircase.finished==False:
                this_intensity = self.staircase.next()
                self.thisTrialN+=1
                corr_resp = 'diff'
                self.count_diff += 1
                trialno = self.count_diff
            elif self.count_same < self.ntrials_same:
                this_intensity = 0
                corr_resp = 'same'
                self.count_same += 1
                trialno = self.count_same
            else:
                self.staircase._terminate()


        this_trial = [
            ('session', self.session),
            ('trialno', trialno),
            ('pos', self.staircase.name),
            ('dir', ori_dir),
            ('oridiff', this_intensity),
            ('onset', ''),
            ('dur', ''),
            ('corr_resp', corr_resp),
            ('subj_resp', ''),
            ('accuracy', ''),
            ('rt', ''),
            ]
        if self.rp['autorun'] > 0:
            auto = self.get_autorun_vals(self.staircase.name,
                    this_intensity, corr_resp)
            this_trial.extend(auto)

        self.this_trial = OrderedDict(this_trial)
        self.exp_plan.append(self.this_trial)
        return self.this_trial

    def get_autorun_vals(self, posno, this_intensity, corr_resp):
        """If autorun, simulate user responses
        """
        def rt(mean):
            add = np.random.normal(mean,scale=.2)/self.rp['autorun']
            return trial_dur + add

        invert_resp = exp.invert_dict(self.computer.valid_responses)
        trial_dur = sum([ev.dur for ev in self.trial])

        if corr_resp == 'same':
            fake_resp = 'same'  # participants should be perfect here
        else:
            thres = np.random.normal(self.convergeto[posno], scale=1)
            thres = max(0, thres)
            if this_intensity < thres:
                fake_resp = 'same'
            else:
                fake_resp = 'diff'

        auto = [('autoresp', invert_resp[fake_resp]),
                ('autort', rt(0.500))]
        return auto

    def create_exp_plan(self):
        self.exp_plan = []  # will be filled up during the runtime

    def get_blocks(self):
        blocks = []
        for name in self.oris.keys():
            blocks.append(data.QuestHandler(
                15, 3,
                pThreshold=.63,
                nTrials=self.ntrials_diff,
                minVal=0,
                maxVal=25,
                name=name
                ))
        self.blocks = blocks

    def post_trial(self, this_trial, all_keys):
        this_resp = all_keys.pop()
        this_trial['subj_resp'] = self.computer.valid_responses[this_resp[0]]
        acc = exp.signal_det(this_trial['corr_resp'], this_trial['subj_resp'])
        this_trial['accuracy'] = acc
        this_trial['rt'] = this_resp[1]
        if acc == 'correct':
            acc_int = 1
        else:
            acc_int = 0
        if this_trial['corr_resp'] == 'diff':
            self.staircase.addData(acc_int)
        return this_trial

    def run_task(self):
        self.setup_task()
        self.before_task()

        if not self.rp['no_output']:
            print 'Data will be saved in %s' % self.datafile.filename
        self.datafile.open()
        for blockno, block in enumerate(self.blocks):
            self.staircase = block
            self.this_blockn = blockno
            self.count_diff = 0
            self.count_same = 0
            self.set_TrialHandler(self.exp_plan)
            self.run_block()
        self.datafile.close()

        self.after_task()


class _Test2AFC(_BaseTest):

    def __init__(self, *args, **kwargs):
        super(_Test2AFC, self).__init__(*args, **kwargs)
        self.blockcol = 'pos'
        self.ntrials = 32  # must be a multiple of 4; very short, just for demo
        self.oridiff = 13

        self.anl = Analysis(info=self.parent.info)

    def create_exp_plan(self):
        exp_plan = []
        for name in self.oris.keys():
            for ori_dir in [-1,1]:
                for trial in range(self.ntrials/2):
                    if trial < self.ntrials/4:
                        corr_resp = 'same'
                        oridiff = 0
                    else:
                        corr_resp = 'diff'
                        oridiff = self.oridiff
                    exp_plan.append(OrderedDict([
                        ('session', self.session),
                        ('pos', name),
                        ('dir', ori_dir),
                        ('oridiff', oridiff),
                        ('onset', ''),
                        ('dur', ''),
                        ('corr_resp', corr_resp),
                        ('subj_resp', ''),
                        ('accuracy', ''),
                        ('rt', ''),
                        ]))
        self.exp_plan = exp_plan

    def set_autorun(self, exp_plan):
        def rt(mean):
            add = np.random.normal(mean,scale=.2) / self.rp['autorun']
            return trial_dur + add

        invert_resp = exp.invert_dict(self.computer.valid_responses)
        trial_dur = sum([ev.dur for ev in self.trial])

        for trial in exp_plan:
            if trial['pos'] == 'attended':
                acc = .9
            elif trial['pos'] == 'unattended':
                acc = .6

            if trial['corr_resp'] == 'same':
                resp_ind = exp.weighted_choice(choices=invert_resp.keys(),
                                                weights=[1-acc, acc])
            else:
                resp_ind = exp.weighted_choice(choices=invert_resp.keys(),
                                                weights=[acc, 1-acc])
            trial['autoresp'] = invert_resp[resp_ind]
            trial['autort'] = rt(.8)
        return exp_plan

    def after_task(self):
        acc = self.anl.test_feedback(self.exp_plan)
        pause_instr = 'Your accuracy is %d%%.' % acc
        super(_Test2AFC, self).after_task(text=pause_instr)


class Analysis(object):
    def __init__(self,
                 name='analysis',
                 info={'subjid': 'perclearn_'},
                 rp=OrderedDict([('no_output', False),
                                 ('plot', True),
                                 ('saveplot', False),
                                 ('all', False),
                                ])
                ):
        self.name = name
        self.info = info
        self.rp = rp
        self.paths = PATHS

        if self.rp['all']:
            self._set_all_subj()

    def _set_all_subj(self):
        self.info['subjid'] = ['perclearn_%02d' % i for i in range(1,11)]

    def plot(self, agg):
        plt = plot.Plot()
        plt.plot(agg)
        if self.rp['plot']:
            plt.show()
        else:
            print agg

    def _train_acc(self, df):
        resp_win = 5  # time window when a response is counted as correct
        resp = df.subj_resp.copy()
        resp[resp.isnull()] = 0
        resp = np.array(resp)
        resp[resp==''] = 0
        count = 0
        for i in range(resp_win):
            roll = np.roll(resp, -i)
            roll[len(roll)-i:] = 0
            count += roll
        df['count'] = count
        df['accuracy'] = ''
        for idx, row in df.iterrows():
            if row['count'] >= 1:
                subj_resp = 1
            else:
                subj_resp = ''
            if row['corr_resp'] == '' or pandas.isnull(row['corr_resp']):
                corr_resp = ''
            else:
                corr_resp = row['corr_resp']
            acc = exp.signal_det(corr_resp, subj_resp)
            if acc == 'false alarm':
                acc = ''
            df.loc[idx,'accuracy'] = acc
        return df

    def train(self):
        pattern = self.paths['data'] + '%s_train.csv'
        df = exp.get_behav_df(self.info['subjid'], pattern=pattern)
        dfs = []
        for session in df.session.unique():
            sdf = df[df.session == session]
            dfs.append(self._train_acc(sdf))
        df = pandas.concat(dfs)
        agg_acc = stats.accuracy(df, cols='session',
                                 values='accuracy', yerr='subjid',
                                 incorrect='miss')
        self.plot(agg_acc)

    def train_feedback(self, session):
        pattern = self.paths['data'] + '%s_train.csv'
        try:
            df = exp.get_behav_df(self.info['subjid'], pattern=pattern)
        except:
            return 0
        else:
            #df = pandas.DataFrame(trial_list)
            df = df[df.session == session]
            df = self._train_acc(df)
            cor = np.sum(df.accuracy == 'correct')
            acc = float(cor) / (np.sum(df.accuracy == 'miss') + cor)
            return acc

    def test_feedback(self, trial_list):
        """Provides feedback during the test phase (2AFC task)
        """
        df = pandas.DataFrame(trial_list)
        acc = float(np.sum(df.accuracy == 'correct'))
        acc /= (np.sum(df.accuracy == 'correct') + \
                np.sum(df.accuracy == 'incorrect'))
        return acc*100

    def test(self):
        """Analysis of the test phase data (for 2AFC task)
        """
        pattern = self.paths['data'] + '%s_test.csv'
        df = exp.get_behav_df(self.info['subjid'], pattern=pattern)
        agg_acc = stats.accuracy(df, rows='session', cols='pos',
            values='accuracy', yerr='subjid')
        plt = plot.Plot()
        plt.plot(agg_acc)
        plt.show()

    def _get_staircase(self):
        pattern = self.paths['data'] + '%s_test.csv'
        df = exp.get_behav_df(subjid=self.info['subjid'], pattern=pattern)
        df = df[df.corr_resp == 'diff']
        df_thres_all = []
        df['thres'] = 0.
        gr = PercLearn(info=self.info)

        for subj in np.unique(df.subjid):
            subjdf = df[df.subjid == subj]
            for sid in subjdf.session.unique():
                sdf = subjdf[subjdf.session == sid]
                for pos in sdf.pos.unique():
                    posdf = sdf[sdf.pos == pos]
                    st_mod = _TestQuest(gr)
                    st_mod.get_blocks()
                    # get staircases from st_mod
                    exps = dict((st.name, st) for st in st_mod.blocks)
                    exps[pos].importData(posdf.oridiff.astype(float),
                                          np.array(posdf.corr_resp==posdf.subj_resp))
                    posdf.thres = exps[pos].mean()
                    df_thres_all.append(posdf)
        df_thres = pandas.concat(df_thres_all)
        return df_thres

    def thresholds(self):
        df = self._get_staircase()
        agg_thres = stats.aggregate(df, values='thres',
            cols='pos', rows='session', yerr='subjid')
        plt = plot.Plot()
        plt.plot(agg_thres, kind='bar', title='orientation thresholds',
                 ylabel='orientation threshold, deg')
        print agg_thres.mean()
        if self.rp['plot']:
            plt.show()

        return agg_thres

    def staircase(self):
        df = self._get_staircase()
        agg_thres = stats.aggregate(df, values='thres',
                rows='session', unstack=True)
        agg = stats.aggregate(df, values='oridiff', subplots='session',
                rows='trialno', cols='pos')
        if self.rp['plot'] or self.rp['saveplot']:
            plt = plot.Plot(nrows=2, ncols=2, sharex=True, sharey=True)
            ax = plt.plot(agg, kind='line', ylabel='step')
            num_trials = len(agg.columns.levels[1])

            for pno, (sno, thres) in enumerate(agg_thres.mean().iteritems()):
                #import pdb; pdb.set_trace()
                ax = plt.get_ax(pno)
                ax.set_title('session %d' % sno)
                ax.axhline(y=thres, color = '.2', ls='--', lw=2,
                           marker = 'None')
                print thres
                ax.text(num_trials - 15, thres+.5,
                    'average threshold = %.2f' % thres, fontsize=8)

            print agg_thres
            plt.hide_plots(3)
            if self.rp['plot']: plt.show()
            if self.rp['saveplot']:
                plt.savefig(PATHS['analysis'] + '%s.svg' % self.info['subjid'])

        return agg_thres





#! /usr/bin/env python
import sys
try:
    import psychopy_ext
except:  # if psychopy_ext not in python path, e.g., you cloned the repo
    try:  # you want to start a new project based on demos
        sys.path.insert(0, '../psychopy_ext')
        import psychopy_ext
    except:  # you're just running demos
        sys.path.insert(0, '../../')
from psychopy_ext import ui

__author__ = "Jonas Kubilius"
__version__ = "0.1"
exp_choices = [
    ui.Choices('scripts.main', name='Simple exp.', alias='main', order=['exp','analysis']),
    ui.Choices('scripts.twotasks', name='Two tasks', order=['exp','analysis']),
    ui.Choices('scripts.staircase', name='Staircase', order=['exp','analysis']),
    ui.Choices('scripts.perclearn', name='Advanced', order=['exp','analysis']),
    ui.Choices('scripts.fmri', name='fMRI')
    ]

# bring up the graphic user interface or interpret command line inputs
# usually you can skip the size parameter
ui.Control(exp_choices, title='Demo Project', size=(560,550))



import sys
import numpy as np

sys.path.insert(0, '../../')
from psychopy_ext import stats, plot

n = 8
nsampl = 10
k = n*nsampl
df = stats.df_fromdict(
    [('subplots', ['session1', 'session2']),
     ('cond', [1, 2]),
     ('name', ['condition one', 'condition two']),
     ('levels', ['small', 'medium', 'large']),
     ('subjID', ['subj%d' % (i+1) for i in range(n)])],
     repeat=nsampl
    )
df['rt'] = 0.5
df.rt[df.cond==1] = np.random.random(12*k) * 1.2
df.rt[df.cond==2] = np.random.random(12*k)
#df['accuracy'] = ['correct','correct','incorrect','incorrect']*k*9

agg = stats.aggregate(df, subplots='subplots', rows='levels',
            cols='cond', yerr='subjID', values='rt')
plt1 = plot.Plot(figsize=(6,3))
plt1.plot(agg, kind='bar')
plt1.tight_layout()

plt2 = plot.Plot(figsize=(6,3))
plt2.plot(agg, kind='line')
plt2.tight_layout()

plt3 = plot.Plot(figsize=(8,4))
plt3.plot(agg, kind='bean')
plt3.tight_layout()

agg = stats.aggregate(df, subplots='subplots', rows='subjID',
            cols='name', values='rt')
plt4 = plot.Plot(figsize=(6,3))
plt4.plot(agg, kind='scatter')
plt4.tight_layout()

mtx = stats.df_fromdict(
    [('subplots', ['session1', 'session2']),
     ('cond1', ['cond%d' % i for i in range(6)]),
     ('cond2', ['cond%d' % i for i in range(6)]),
     ('subjID', ['subj%d' % (i+1) for i in range(n)])],
     repeat=nsampl
    )
mtx['corr'] = np.random.random(len(mtx))
agg = stats.aggregate(mtx, subplots='subplots', rows='cond1',
            cols='cond2', values='corr')
plt5 = plot.Plot(figsize=(8,4), kind='matrix')
plt5.plot(agg, kind='matrix')

#agg = stats.aggregate(df, subplots='subplots', rows='cond',
            #cols='levels', yerr='subjID', values='rt')
#agg = stats.aggregate(df, subplots='subplots', rows=['cond', 'name'],
            #cols='levels', yerr='subjID', values='rt')


plt5.show()



from distutils.core import setup


description = ('A framework for a rapid reproducible design, analysis and '
               'plotting of  experiments in neuroscience and psychology.')
exec(open('psychopy_ext/version.py').read())

setup(
    name='psychopy_ext',
    version=__version__,
    author='Jonas Kubilius',
    author_email='qbilius@gmail.com',
    packages=['docs', 'psychopy_ext', 'psychopy_ext.demos', 
              'psychopy_ext.demos.scripts', 'psychopy_ext.tests'],
    package_data={'': ['*.png', '*.py*', '*.bat', '*.csv', '*.txt']},
    url='https://github.com/qbilius/psychopy_ext/',
    license='GNU General Public License v3 or later',
    description=description,
    long_description=open('README.rst').read(),
    install_requires=[
        "psychopy >= 1.7",
        "pandas >= 0.12",
        "pymvpa2 >= 2.0",
        "docutils"
    ],
    #test_suite='nose.collector',
    #tests_require=['nose'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering'
    ],
    keywords=('psychology experiments plotting data visualization analysis ',
              'fMRI simulations hmax gaborjet reproducible research open science'),
)



import numpy as np
from .. import models

import unittest

class TestHMAX(unittest.TestCase):
    def test_gaussian(self):
        m = models.HMAX(matlab=True, filt_type='gaussian')
        out = m.run()
        fid = open('psychopy_ext/tests/lena_gaussian_matlab.txt')
        c2_matlab = np.array([float(i.strip('\n')) for i in fid.readlines()])
        c2_python = np.around(out['C2'], decimals=5)  # matlab's output has 5
                                                      # significant digits
        rms = np.mean(np.sqrt((c2_matlab - c2_python)**2))
        self.assertEqual(rms, 0)

    def test_gabor(self):
        m = models.HMAX(matlab=False, filt_type='gabor')
        out = m.run()
        fid = open('psychopy_ext/tests/lena_gabor_matlab.txt')
        c2_matlab = np.array([float(i.strip('\n')) for i in fid.readlines()])
        c2_python = np.around(out['C2'], decimals=5)  # matlab's output has 5
                                                      # significant digits
        rms = np.mean(np.sqrt((c2_matlab - c2_python)**2))
        self.assertEqual(rms, 0)


class TestGaborJets(unittest.TestCase):
    def setUp(self):
        m = models.GaborJet()
        self.mag, self.phase, self.grid = m.run(m.get_teststim())

    def test_mag(self):
        mag_matlab = np.genfromtxt('psychopy_ext/tests/jet_mag.txt', delimiter=',')
        mag_matlab = mag_matlab.ravel()
        mag_python = np.around(self.mag, decimals=5)  # matlab's output has 5
                                                      # significant digits
        mag_python = mag_python.ravel()
        rms = np.mean(np.sqrt((mag_matlab - mag_python)**2))
        self.assertTrue(rms, 0)

    def test_phase(self):
        phase_matlab = np.genfromtxt('psychopy_ext/tests/jet_phase.txt',
                                     delimiter=',')
        phase_matlab = phase_matlab.ravel()
        phase_python = np.around(self.mag, decimals=5) # matlab's output has 5
                                                       # significant digits
        phase_python = phase_python.ravel()
        rms = np.mean(np.sqrt((phase_matlab - phase_python)**2))
        self.assertTrue(rms, 0)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import pandas
from .. import stats, plot

import unittest

N = 8
NSAMPL = 10

class TestAgg(unittest.TestCase):

    def get_df(self, n=N, nsampl=NSAMPL):
        k = n * nsampl
        df = stats.df_fromdict(
            [('subplots', ['session1', 'session2']),
            ('cond', [1, 2]),
            ('name', ['one', 'two', 'three']),
            ('levels', ['small', 'medium', 'large']),
            ('subjID', ['subj%d' % (i+1) for i in range(8)])],
            repeat=10
            )
        df['rt'] = range(k)*36
        df['accuracy'] = ['correct','correct','incorrect','incorrect']*k*9
        return df

    def test_df_fromdict(self):
        n = 8
        nsampl = 10
        df = self.get_df(n=n, nsampl=nsampl)
        k = n * nsampl
        data = {
            'subplots': ['session1']*k*18 + ['session2']*k*18,
            'cond': [1]*k*9 + [2]*k*9 + [1]*k*9 + [2]*k*9,
            'name': (['one', 'one', 'one']*k + ['two', 'two', 'two']*k +
                    ['three', 'three', 'three']*k) * 4,
            'levels': (['small']*k + ['medium']*k + ['large']*k)*12,
            'subjID': ['subj%d' % (i+1) for i in np.repeat(range(n),nsampl)] * 36,
            'rt': range(k)*36,
            'accuracy': ['correct','correct','incorrect','incorrect']*k*9
            }
        df_manual = pandas.DataFrame(data, columns = ['subplots','cond','name',
            'levels','subjID','rt','accuracy'])
        self.assertEqual(df.to_string(), df_manual.to_string())

    def test_aggregate_random(self):
        df = self.get_df()
        df.rt = df.rt.astype(float)
        df['rt'] = np.genfromtxt('psychopy_ext/tests/rt.csv', delimiter=',')
        agg = stats.aggregate(df, subplots='subplots', rows=['cond', 'name'],
            cols='levels', yerr='subjID', values='rt')

        # construct the correct answer
        tuples = zip(*[
            ['session1']*18 + ['session2']*18,
            [1]*9 + [2]*9 + [1]*9 + [2]*9,
            ['one', 'one', 'one', 'two', 'two', 'two','three', 'three', 'three'] * 4,
            ['small','medium','large']*12
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','rows.name','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        data = np.genfromtxt('psychopy_ext/tests/agg.csv', delimiter=',')
        test_agg = pandas.DataFrame(data, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate(self):
        df = self.get_df()
        agg = stats.aggregate(df, subplots='subplots', rows=['cond', 'name'],
            cols='levels', yerr='subjID', values='rt')

        # construct the correct answer
        col = [4.5, 14.5, 24.5, 34.5, 44.5, 54.5, 64.5, 74.5]
        tuples = zip(*[
            ['session1']*18 + ['session2']*18,
            [1]*9 + [2]*9 + [1]*9 + [2]*9,
            ['one', 'one', 'one', 'two', 'two', 'two','three', 'three', 'three'] * 4,
            ['small','medium','large']*12
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','rows.name','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*36, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate_nosubplots(self):
        df = self.get_df()
        agg = stats.aggregate(df, rows=['cond', 'name'],
            cols='levels', yerr='subjID', values='rt')

        # construct the correct answer
        col = [4.5, 14.5, 24.5, 34.5, 44.5, 54.5, 64.5, 74.5]
        tuples = zip(*[
            [1]*9 + [2]*9,
            ['one', 'one', 'one', 'two', 'two', 'two','three', 'three', 'three'] * 2,
            ['small','medium','large']*6
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['rows.cond','rows.name','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*18, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate_onerow(self):
        df = self.get_df()
        agg = stats.aggregate(df, subplots='subplots', rows='cond',
            cols='levels', yerr='subjID', values='rt')

        # construct the correct answer
        col = [4.5, 14.5, 24.5, 34.5, 44.5, 54.5, 64.5, 74.5]
        tuples = zip(*[
            ['session1']*6 + ['session2']*6,
            [1]*3 + [2]*3 + [1]*3 + [2]*3,
            ['small','medium','large']*6
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*12, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate_nocols(self):
        df = self.get_df()
        agg = stats.aggregate(df, subplots='subplots', rows=['cond', 'name'],
            yerr='subjID', values='rt')

        # construct the correct answer
        col = [4.5, 14.5, 24.5, 34.5, 44.5, 54.5, 64.5, 74.5]
        tuples = zip(*[
            ['session1']*6 + ['session2']*6,
            [1]*3 + [2]*3 + [1]*3 + [2]*3,
            ['one', 'two', 'three'] * 4
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','rows.name'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*12, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        #import pdb; pdb.set_trace()
        #gg
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate_norows(self):
        df = self.get_df()
        agg = stats.aggregate(df, subplots='subplots',
            cols='levels', yerr='subjID', values='rt')

        # construct the correct answer
        col = [4.5, 14.5, 24.5, 34.5, 44.5, 54.5, 64.5, 74.5]
        tuples = zip(*[
            ['session1']*3 + ['session2']*3,
            ['small','medium','large']*2
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*6, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_aggregate_noyerr(self):
        df = self.get_df()
        agg = stats.aggregate(df, subplots='subplots', rows=['cond', 'name'],
            cols='levels', values='rt')

        # construct the correct answer
        col = [39.5]
        tuples = zip(*[
            ['session1']*18 + ['session2']*18,
            [1]*9 + [2]*9 + [1]*9 + [2]*9,
            ['one', 'one', 'one', 'two', 'two', 'two','three', 'three', 'three'] * 4,
            ['small','medium','large']*12
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','rows.name','cols.levels'])
        test_agg = pandas.DataFrame([col]*36, index=index).T
        test_agg = test_agg.rename(index={0: 'rt'})
        self.assertEqual(test_agg.to_string(), agg.to_string())

    def test_accuracy(self):
        df = self.get_df()
        agg = stats.accuracy(df, subplots='subplots', rows=['cond', 'name'],
            cols='levels', yerr='subjID', values='accuracy')

        # construct the correct answer
        col = [.6, .4] * (N/2)
        tuples = zip(*[
            ['session1']*18 + ['session2']*18,
            [1]*9 + [2]*9 + [1]*9 + [2]*9,
            ['one', 'one', 'one', 'two', 'two', 'two','three', 'three', 'three'] * 4,
            ['small','medium','large']*12
            ])
        index = pandas.MultiIndex.from_tuples(tuples,
            names=['subplots.subplots','rows.cond','rows.name','cols.levels'])
        cols = pandas.Index(['subj%d' % (i+1) for i in range(N)], names='yerr.subjID')
        test_agg = pandas.DataFrame([col]*36, index=index, columns=cols).T
        test_agg.index.names = ['yerr.subjID']  # yeah...
        self.assertEqual(test_agg.to_string(), agg.to_string())

if __name__ == '__main__':
    unittest.main()



"""
An simple example of an experiment with two tasks.

This demo is part of psychopy_ext library.
"""

import sys
import cPickle as pickle

from psychopy import core, visual, data, event
import numpy as np
# pandas does not come by default with PsychoPy but that should not prevent
# people from running the experiment
try:
    import pandas
except:
    pass

from psychopy_ext import exp, ui, stats, plot

# some modules are only available in Python 2.6
try:
    from collections import OrderedDict
except:
    from exp import OrderedDict

import computer
PATHS = exp.set_paths('twotasks', computer)


class TwoTasks(exp.Experiment):
    """
    A perceptual learning experiment
    ================================

    This experiment is composed of four parts. Each part consists of
    **training** for 1 min (for this demo purposes only) and
    **testing** for 5 min.

    **Press spacebar to continue.**

    *(Use 'Left Shift + Esc' to exit.)*
    """
    def __init__(self,
        name='exp',
        info=('subjid', 'twotasks_'),
        rp=('phase', ('both', 'train', 'test'))
        ):
        super(TwoTasks, self).__init__(name=name, info=info, rp=rp,
                actions='run', paths=PATHS, computer=computer)

        self.nsessions = 3 # number of sessions
        self.tasks = [_Train, _Test]

        # stimuli orientation
        self.oris = {'attended': 60, 'unattended': 150}
        self.stim_size = 3.  # in deg
        self.stim_dist = 4.  # from the fixation in polar coords

    def run(self):
        """
        This is a more complicated run method than usually because
        I wanted to demonstrate how to have a few sessions of the
        same few tasks. If you don't need this, you can simply comment
        out this method and it will still run one session (note that
        ``phase argument will no longer work``).
        """
        self.setup()
        self.before_exp()
        for expno in range(self.nsessions):
            self.show_text('Session %d' % (expno+1), auto=1)
            if self.rp['phase'] in ['train','both']:
                self.tasks[0](self, session=expno+1).run_task()
            if self.rp['phase'] in ['test', 'both']:
                self.tasks[1](self, session=expno+1).run_task()
            pause_instr = ('End of session %d.\n'
                           'When ready, press spacebar to continue.'
                            % (expno+1))
            if expno != self.nsessions - 1:
                self.show_text(pause_instr)
        self.after_exp()

        if self.rp['register']:
            self.register()
        elif self.rp['push']:
            self.commitpush()
        self.quit()

class _Train(exp.Task):
    """
    Training
    ========

    Your task
    ---------

    **Fixate** on the central fixation spot.
    Attend to stimulus on the **left**.
    When you notice a decrease in its contrast, **press 'j'**.
    Please remember to fixate throughout the experiment.

    **Press spacebar to begin.**

    *(Use 'Left Shift + Esc' to exit.)*
    """

    def __init__(self, parent, session=1):

        data_fname = parent.paths['data'] + parent.info['subjid'] + '_train.csv'
        super(_Train, self).__init__(
            parent,
            method='random',
            data_fname=data_fname
            )
        self.session = session
        self.computer.valid_responses = {'j': 1}

        self.oris = parent.oris
        self.stim_size = parent.stim_size
        self.stim_dist = parent.stim_dist

        self.ntrials = 75  # number of trials per session
        self.nblocks = 2
        self.task_rate = .1  # how often lower contrast stimulus should come up
        self.low_contrast = .5

    def create_stimuli(self):
        # Define stimuli
        self.create_fixation()
        self.s = {'fix': self.fixation}
        self.s['attended'] = visual.GratingStim(self.win, name='attended',
                                mask='circle', sf=2, size=self.stim_size,
                                pos=(-self.stim_dist,0),
                                ori=self.oris['attended'])
        self.s['unattended'] = visual.GratingStim(self.win, name='unattended',
                                mask='circle', sf=2, size=self.stim_size,
                                pos=(self.stim_dist,0),
                                ori=self.oris['unattended'])

    def create_trial(self):
        """Create trial structure
        """
        allstim = [self.s['fix'], self.s['attended'], self.s['unattended']]

        self.trial = [exp.Event(self,
                                dur=.200,  # in seconds
                                display=allstim,
                                func=self.show_stim),
                      exp.Event(self,
                                dur=.200,
                                display=self.s['fix'],
                                func=self.idle_event)
                     ]

    def create_exp_plan(self):
        """Define each trial's parameters
        """
        exp_plan = []
        for block in range(self.nblocks):
            for trial in range(self.ntrials):  # repeat the defined number of times
                if trial < self.ntrials * self.task_rate:
                    cond = 'low'
                    corr_resp = 1
                else:
                    cond = 'high'
                    corr_resp = ''
                exp_plan.append(OrderedDict([
                    ('session', self.session),
                    ('block', block),
                    ('cond', cond),
                    ('onset', ''),
                    ('dur', ''),
                    ('corr_resp', corr_resp),
                    ('subj_resp', ''),
                    ('rt', ''),
                    ]))
        self.exp_plan = exp_plan

    def show_stim(self):
        if self.this_trial['cond'] == 'low':
            contrast = self.low_contrast
        else:
            contrast = 1
        for stim in self.this_event.display:
            if stim.name == 'attended':
                stim.setContrast(contrast)
            stim.draw()
        self.win.flip()

        event_keys = self.idle_event(draw_stim=False)

        return event_keys

    def post_trial(self, this_trial, all_keys):
        """ What to do after a trial is over.
        """
        if len(all_keys) > 0:
            if all_keys[0][0] != '':
                this_trial['subj_resp'] = len(all_keys)
            else:
                this_trial['subj_resp'] = ''
            this_trial['rt'] = all_keys[-1][1]
        else:
            this_trial['subj_resp'] = ''
            this_trial['rt'] = ''

        return this_trial

    def before_task(self):
        """We slightly redefine the default function so that full
        instructions are shown the first time round.
        """
        if self.session == 1:
            super(_Train, self).before_task()
        else:
            text = '''
            Training, session %d
            --------------------

            (decrease in its contrast: **press 'j'**)
            '''
            super(_Train, self).before_task(text=text % self.session)


class _Test(exp.Task):
    """
    Testing
    =======

    Your task
    ---------

    **Fixate** on the central fixation spot. Two stimuli will briefly flash
    in a row.
    If the second stimulus is oriented to the left with respect to the
    first one, **hit 'f'**.
    If the second stimulus is oriented to the right with respect to the
    first one, **hit 'j'**.
    Please remember to fixate throughout the experiment!

    **Press spacebar to begin.**

    *(Use 'Left Shift + Esc' to exit.)*
    """
    def __init__(self, parent, session=1):
        data_fname = (parent.paths['data'] + parent.info['subjid'] + '_test.csv')
        super(_Test, self).__init__(
            parent,
            method='random',
            blockcol = 'pos',
            data_fname=data_fname
            )
        self.session = session
        self.oris = parent.oris
        self.stim_size = parent.stim_size
        self.stim_dist = parent.stim_dist

        self.ntrials = 32  # must be a multiple of 4; very short, just for demo
        self.oridiff = 13

        self.computer.valid_responses = {'f': 'same', 'j': 'diff'}

        self.anl = Analysis(info=self.parent.info)

    def create_stimuli(self):
        self.create_fixation()
        stim1 = visual.GratingStim(self.win, name='stim1', mask='circle',
                                        sf=2, size=self.stim_size)
        stim2 = visual.GratingStim(self.win, name='stim2', mask='circle',
                                        sf=2, size=self.stim_size)
        self.s = {'fix': self.fixation, 'stim1': stim1, 'stim2': stim2}

    def create_trial(self):
        """Create trial structure
        """
        self.trial = [exp.Event(self,
                                dur=.200,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self,
                                dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim1']],
                                func=self.show_stim),
                      exp.Event(self,
                                dur=.600,
                                display=self.s['fix'],
                                func=self.idle_event),
                      exp.Event(self,
                                dur=.300,  # in seconds
                                display=[self.s['fix'], self.s['stim2']],
                                func=self.show_stim),
                      exp.Event(self,
                                dur=0,
                                display=self.s['fix'],
                                func=self.wait_until_response)
                     ]

    def create_exp_plan(self):
        exp_plan = []
        for name in self.oris.keys():
            for ori_dir in [-1,1]:
                for trial in range(self.ntrials/2):
                    if trial < self.ntrials/4:
                        corr_resp = 'same'
                        oridiff = 0
                    else:
                        corr_resp = 'diff'
                        oridiff = self.oridiff
                    exp_plan.append(OrderedDict([
                        ('session', self.session),
                        ('pos', name),
                        ('dir', ori_dir),
                        ('oridiff', oridiff),
                        ('onset', ''),
                        ('dur', ''),
                        ('corr_resp', corr_resp),
                        ('subj_resp', ''),
                        ('accuracy', ''),
                        ('rt', ''),
                        ]))
        self.exp_plan = exp_plan

    def set_autorun(self, exp_plan):
        def rt(mean):
            add = np.random.normal(mean,scale=.2) / self.rp['autorun']
            return trial_dur + add

        invert_resp = exp.invert_dict(self.computer.valid_responses)
        trial_dur = sum([ev.dur for ev in self.trial])

        for trial in exp_plan:
            if trial['pos'] == 'attended':
                acc = .9
            elif trial['pos'] == 'unattended':
                acc = .6

            if trial['corr_resp'] == 'same':
                resp_ind = exp.weighted_choice(choices=invert_resp.keys(),
                                                weights=[1-acc, acc])
            else:
                resp_ind = exp.weighted_choice(choices=invert_resp.keys(),
                                                weights=[acc, 1-acc])
            trial['autoresp'] = invert_resp[resp_ind]
            trial['autort'] = rt(.8)
        return exp_plan

    def show_stim(self):
        if self.this_trial['pos'] == 'attended':
            pos = (-self.stim_dist, 0)
        else:
            pos = (self.stim_dist, 0)

        for stim in self.this_event.display:
            if stim.name == 'stim1':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']])
            if stim.name == 'stim2':
                stim.setPos(pos)
                stim.setOri(self.oris[self.this_trial['pos']] +
                            self.this_trial['dir']*self.this_trial['oridiff'])
            stim.draw()
        self.win.flip()

        self.idle_event(draw_stim=False)

    def before_task(self):
        """We slightly redefine the default function so that full
        instructions are shown the first time round.
        """
        if self.session == 1:
            super(_Test, self).before_task()
        else:
            text = '''
            Testing, session %d
            -------------------

            (Second stimulus oriented to the:

            - left: **hit 'f'**
            - right: **hit 'j'**)
            '''
            super(_Test, self).before_task(text=text % self.session)

    def after_task(self):
        acc = self.anl.test_feedback(self.exp_plan)
        pause_instr = 'Your accuracy is %d%%.' % acc
        super(_Test, self).after_task(text=pause_instr)


class Analysis(object):
    def __init__(self,
                 name='analysis',
                 info={'subjid': 'twotasks_'},
                 rp={'all': False},
                 actions='test'
                 ):
        self.name = name
        self.info = info
        self.rp = rp
        self.paths = PATHS

        if self.rp['all']:
            self._set_all_subj()

    def _set_all_subj(self):
        self.info['subjid'] = ['twotasks_%02d' % i for i in range(1,11)]

    def test_feedback(self, trial_list):
        """Provides feedback during the test phase
        """
        df = pandas.DataFrame(trial_list)
        acc = float(np.sum(df.accuracy == 'correct'))
        acc /= (np.sum(df.accuracy == 'correct') + \
                np.sum(df.accuracy == 'incorrect'))
        return acc*100

    def test(self):
        """Analysis of the test phase data
        """
        pattern = self.paths['data'] + '%s_test.csv'
        df = exp.get_behav_df(self.info['subjid'], pattern=pattern)
        agg_acc = stats.accuracy(df, rows='session', cols='pos',
            values='accuracy', yerr='subjid')
        plt = plot.Plot()
        plt.plot(agg_acc)
        plt.show()
